@@ -1,25 +1,22 @@
 import numpy as np
+import xarray as xr
+import warnings
 import scipy.signal as ss
 from scipy.integrate import cumtrapz
-import xarray as xr
+
 from ..rotate import vector as rot
-#from ..rotate import signature as sig
 from ..rotate.api import _make_model, rotate2
-import warnings
 
 
 def _get_body2imu(make_model):
     if make_model == 'nortek vector':
         # In inches it is: (0.25, 0.25, 5.9)
         return np.array([0.00635, 0.00635, 0.14986])
-    # Placeholder
-    # elif make_model.startswith('nortek signature'):
-    #     return np.array([0.0, 0.0, 0.0])
     else:
         raise Exception("The imu->body vector is unknown for this instrument.")
 
 
-class _CalcMotion():
+class CalcMotion():
     """
     A 'calculator' for computing the velocity of points that are
     rigidly connected to an ADV-body with an IMU.
@@ -90,43 +87,103 @@ class _CalcMotion():
                              "perform motion correction.")
                             % (self.ds.coord_sys))
 
+    def _check_duty_cycle(self, ):
+        """
+        Function to check if duty cycle exists and if it is followed
+        consistently in the datafile
+
+        """
+        n_burst = self.ds.attrs.get('duty_cycle_n_burst')
+        if not n_burst:
+            return
+
+        # duty cycle interval in seconds
+        interval = self.ds.attrs.get('duty_cycle_interval')
+        actual_interval = (
+            self.ds.time[n_burst:].values - self.ds.time[:-n_burst].values)/1e9
+
+        rng = actual_interval.max() - actual_interval.min()
+        mean = actual_interval.mean()
+        # Range will vary depending on how datetime64 rounds the timestamp
+        # But isn't an issue if it does
+        if rng > 2 or (mean > interval+1 and mean < interval-1):
+            raise Exception("Bad duty cycle detected")
+
+        # If this passes, it means we're save to blindly skip n_burst for every integral
+        return n_burst
+
+    def reshape(self, dat, n_bin):
+        # Assumes shape is (3, time)
+        length = dat.shape[-1]//n_bin
+        return np.reshape(dat[..., :length*n_bin], (dat.shape[0], length, n_bin))
+
     def _set_acclow(self, ):
+        # Check if file is duty cycled
+        n = self._check_duty_cycle()
+
+        if n:
+            warnings.warn("   Duty Cycle detected. "
+                          "Motion corrected data may contain edge effects "
+                          "at the beginning and end of each duty cycle.")
+            self.accel = self.reshape(self.accel, n_bin=n)
+
         self.acclow = acc = self.accel.copy()
         if self.accel_filtfreq == 0:
             acc[:] = acc.mean(-1)[..., None]
         else:
             flt = ss.butter(1, self.accel_filtfreq / (self.ds.fs / 2))
             for idx in range(3):
-                acc[idx] = ss.filtfilt(flt[0], flt[1], acc[idx])
+                acc[idx] = ss.filtfilt(flt[0], flt[1], acc[idx], axis=-1)
 
     def calc_velacc(self, ):
         """
         Calculates the translational velocity from the high-pass
-        filtered acceleration sigal.
+        filtered acceleration signal.
 
         Returns
         -------
-        velacc : |np.ndarray| (3 x n_time)
+        velacc : numpy.ndarray (3 x n_time)
                The acceleration-induced velocity array (3, n_time).
         """
         samp_freq = self.ds.fs
 
+        # Check if file is duty cycled
+        n = self._check_duty_cycle()
+        # accel & accel-low will already be reshaped if n isn't none
+
         # Get high-pass accelerations
         hp = self.accel - self.acclow
 
         # Integrate in time to get velocities
         dat = np.concatenate((np.zeros(list(hp.shape[:-1]) + [1]),
-                              cumtrapz(hp, dx=1 / samp_freq)), axis=-1)
+                              cumtrapz(hp, dx=1 / samp_freq, axis=-1)), axis=-1)
 
-        # Filter again?
         if self.accelvel_filtfreq > 0:
             filt_freq = self.accelvel_filtfreq
             # 2nd order Butterworth filter
             # Applied twice by 'filtfilt' = 4th order butterworth
             filt = ss.butter(2, float(filt_freq) / (samp_freq / 2))
             for idx in range(hp.shape[0]):
-                dat[idx] = dat[idx] - ss.filtfilt(filt[0], filt[1], dat[idx])
-        return dat
+                dat[idx] = dat[idx] - \
+                    ss.filtfilt(filt[0], filt[1], dat[idx], axis=-1)
+
+        if n:
+            # remove reshape
+            velacc_shaped = np.empty(self.angrt.shape)
+            acclow_shaped = np.empty(self.angrt.shape)
+            accel_shaped = np.empty(self.angrt.shape)
+            for idx in range(hp.shape[0]):
+                velacc_shaped[idx] = np.ravel(dat[idx], 'C')
+                acclow_shaped[idx] = np.ravel(self.acclow[idx], 'C')
+                accel_shaped[idx] = np.ravel(self.accel[idx], 'C')
+
+            # return acclow and velacc
+            self.acclow = acclow_shaped
+            self.accel = accel_shaped
+            return velacc_shaped
+
+        else:
+            return dat
 
     def calc_velrot(self, vec, to_earth=None):
         """
@@ -135,14 +192,14 @@ class _CalcMotion():
 
         Parameters
         ----------
-        vec : |np.ndarray| (len(3) or 3 x M)
+        vec : numpy.ndarray (len(3) or 3 x M)
           The vector in meters (or vectors) from the body-origin
           (center of head end-cap) to the point of interest (in the
           body coord-sys).
 
         Returns
         -------
-        velrot : |np.ndarray| (3 x M x N_time)
+        velrot : numpy.ndarray (3 x M x N_time)
           The rotation-induced velocity array (3, n_time).
 
         """
@@ -186,23 +243,35 @@ class _CalcMotion():
 
 def _calc_probe_pos(ds, separate_probes=False):
     """
-    Currently this only works for Nortek Vectors!
+    Calculates the position of probe (or "head") of an ADV.
 
-    In the future, we could use the transformation matrix (and a
-    probe-length lookup-table?)
+    Paratmeters
+    -----------
+    ds : xarray.Dataset
+      ADV dataset
+    separate_probes : bool (optional, default=False)
+      If a Nortek Vector ADV, this function returns the 
+      transformation matrix of positions of the probe's 
+      acoustic recievers to the ADV's instrument frame of
+      reference
+
+    Returns
+    -------
+    vec : 3x3 numpy.ndarray
+      Transformation matrix to convert from ADV probe to 
+      instrument frame of reference
 
     """
-    # According to the ADV_DataSheet, the probe-length radius is
-    # 8.6cm @ 120deg from probe-stem axis.  If I subtract 1cm
-    # to get acoustic receiver center, this is 7.6cm.
-    # In the coordinate sys of the center of the probe
-    # then, the positions of the centers of the receivers is:
-    # if separate_probes and p['inst_make'].lower() == 'nortek' and\
-    #    p['inst_model'].lower == 'vector':
     vec = ds.inst2head_vec
     if type(vec) != np.ndarray:
         vec = np.array(vec)
 
+    # According to the ADV technical drawing, the probe-length radius
+    # is 8.6 cm @ 120 deg from probe-stem axis.  If I subtract 1 cm
+    # to get the center of a acoustic receiver, this is 7.6 cm.
+    # In the coordinate system of the center of the probe (origin at
+    # the acoustic transmitter) then, the positions of the centers of
+    # the receivers is:
     if separate_probes and _make_model(ds) == 'nortek vector':
         r = 0.076
         # The angle between the x-y plane and the probes
@@ -231,7 +300,7 @@ def correct_motion(ds,
     Parameters
     ----------
     ds : xarray.Dataset
-      Cleaned ADV dataset
+      Cleaned ADV dataset in "inst" coordinates
 
     accel_filtfreq : float
       the frequency at which to high-pass filter the acceleration
@@ -331,18 +400,18 @@ def correct_motion(ds,
     rot._check_inst2head_rotmat(ds)
 
     # Create the motion 'calculator':
-    calcobj = _CalcMotion(ds,
-                          accel_filtfreq=accel_filtfreq,
-                          vel_filtfreq=vel_filtfreq,
-                          to_earth=to_earth)
+    calcobj = CalcMotion(ds,
+                         accel_filtfreq=accel_filtfreq,
+                         vel_filtfreq=vel_filtfreq,
+                         to_earth=to_earth)
 
     ##########
     # Calculate the translational velocity (from the accel):
-    ds['velacc'] = xr.DataArray(calcobj.calc_velacc(),
-                                dims=['dirIMU', 'time'])
+    ds['velacc'] = xr.DataArray(calcobj.calc_velacc(), dims=[
+                                'dirIMU', 'time']).astype('float32')
     # Copy acclow to the adv-object.
-    ds['acclow'] = xr.DataArray(calcobj.acclow,
-                                dims=['dirIMU', 'time'])
+    ds['acclow'] = xr.DataArray(
+        calcobj.acclow, dims=['dirIMU', 'time']).astype('float32')
 
     ##########
     # Calculate rotational velocity (from angrt):
@@ -367,10 +436,8 @@ def correct_motion(ds,
                                                  velrot)))
         # 5) Rotate back to body-coord.
         velrot = np.dot(rmat.T, velrot)
-    # try:
-    ds['velrot'] = xr.DataArray(velrot, dims=['dirIMU', 'time'])
-    # except:
-    #    ds['velrot'] = xr.DataArray(velrot, dims=['dirIMU','range','time'])
+    ds['velrot'] = xr.DataArray(
+        velrot, dims=['dirIMU', 'time']).astype('float32')
 
     ##########
     # Rotate the data into the correct coordinate system.
@@ -384,11 +451,9 @@ def correct_motion(ds,
 
     # NOTE: accel, acclow, and velacc are in the earth-frame after
     #       calc_velacc() call.
-    # if _make_model(ds).startswith('nortek signature'):
-    #     inst2earth = sig._inst2earth
-    # else: # nortek vector
     inst2earth = rot._inst2earth
     if to_earth:
+        # accel was converted to earth coordinates
         ds['accel'].values = calcobj.accel
         to_remove = ['accel', 'acclow', 'velacc']
         ds = inst2earth(ds, rotate_vars=[e for e in
@@ -397,17 +462,18 @@ def correct_motion(ds,
     else:
         # rotate these variables back to the instrument frame.
         ds = inst2earth(ds, reverse=True,
-                        rotate_vars=['accel', 'acclow', 'velacc'],
+                        rotate_vars=['acclow', 'velacc'],
                         force=True)
 
     ##########
     # Copy vel -> velraw prior to motion correction:
-    ds['vel_raw'] = xr.DataArray(ds.vel.copy(deep=True), dims=ds.vel.dims)
+    ds['vel_raw'] = xr.DataArray(ds.vel.copy(
+        deep=True), dims=ds.vel.dims).astype('float32')
     # Add it to rotate_vars:
     ds.attrs['rotate_vars'].append('vel_raw')
 
     ##########
-    # Remove motion from measured velocity! <woot!>
+    # Remove motion from measured velocity
     # NOTE: The plus sign is because the measured-induced velocities
     #       are in the opposite direction of the head motion.
     #       i.e. when the head moves one way in stationary flow, it
@@ -416,13 +482,6 @@ def correct_motion(ds,
     # use xarray to keep dimensions consistent
     velmot = ds['velrot'] + ds['velacc']
     velmot = velmot.values
-
-    # if _make_model(ds).startswith('nortek signature'):
-    #     # drop xarray to not break code between ADV and AD2CP
-    #     ds['vel'][:3] += velmot[:,None,:]
-    #     # This assumes these are w.
-    #     ds['vel'][3:] += velmot[2:]
-    # else: # nortek vector
     ds['vel'][:3] += velmot
 
     ds.attrs['motion corrected'] = 1
