@@ -2,15 +2,20 @@ import numpy as np
 import xarray as xr
 from struct import unpack, calcsize
 import warnings
+from pathlib import Path
+import logging
+
 from . import nortek2_defs as defs
 from . import nortek2_lib as lib
-from .base import _find_userdata, _create_dataset
+from .base import _find_userdata, _create_dataset, _abspath
 from ..rotate.vector import _euler2orient
 from ..rotate.base import _set_coords
 from ..rotate.api import set_declination
+from ..time import epoch2dt64, _fill_time_gaps
 
 
-def read_signature(filename, userdata=True, nens=None):
+def read_signature(filename, userdata=True, nens=None, rebuild_index=False,
+                   debug=False, **kwargs):
     """Read a Nortek Signature (.ad2cp) datafile
 
     Parameters
@@ -27,7 +32,19 @@ def read_signature(filename, userdata=True, nens=None):
     -------
     ds : xarray.Dataset
         An xarray dataset from the binary instrument data
+
     """
+    # Start debugger logging
+    if debug:
+        for handler in logging.root.handlers[:]:
+            logging.root.removeHandler(handler)
+        filepath = Path(filename)
+        logfile = filepath.with_suffix('.log')
+        logging.basicConfig(filename=str(logfile),
+                            filemode='w',
+                            level=logging.NOTSET,
+                            format='%(name)s - %(levelname)s - %(message)s')
+
     if nens is None:
         nens = [0, None]
     else:
@@ -42,12 +59,26 @@ def read_signature(filename, userdata=True, nens=None):
 
     userdata = _find_userdata(filename, userdata)
 
-    rdr = _Ad2cpReader(filename)
+    rdr = _Ad2cpReader(filename, rebuild_index=rebuild_index, debug=debug)
     d = rdr.readfile(nens[0], nens[1])
     rdr.sci_data(d)
     out = _reorg(d)
     _reduce(out)
 
+    # Convert time to dt64 and fill gaps
+    coords = out['coords']
+    t_list = [t for t in coords if 'time' in t]
+    for ky in t_list:
+        tdat = coords[ky]
+        tdat[tdat == 0] = np.NaN
+        if np.isnan(tdat).any():
+            tag = ky.lstrip('time')
+            warnings.warn("Zero/NaN values found in '{}'. Interpolating and "
+                          "extrapolating them. To identify which values were filled later, "
+                          "look for 0 values in 'status{}'".format(ky, tag))
+            tdat = _fill_time_gaps(tdat, sample_rate_hz=out['attrs']['fs'])
+        coords[ky] = epoch2dt64(tdat).astype('datetime64[us]')
+
     declin = None
     for nm in userdata:
         if 'dec' in nm:
@@ -67,19 +98,27 @@ def read_signature(filename, userdata=True, nens=None):
                                                'time': ds['time']},
                                        dims=['earth', 'inst', 'time'])
     if declin is not None:
-        ds = set_declination(ds, declin)
+        set_declination(ds, declin, inplace=True)
+
+    # Close handler
+    if debug:
+        for handler in logging.root.handlers[:]:
+            logging.root.removeHandler(handler)
+            handler.close()
 
     return ds
 
 
 class _Ad2cpReader():
-    def __init__(self, fname, endian=None, bufsize=None, rebuild_index=False):
+    def __init__(self, fname, endian=None, bufsize=None, rebuild_index=False,
+                 debug=False):
         self.fname = fname
         self._check_nortek(endian)
         self.f.seek(0, 2)  # Seek to end
         self._eof = self.f.tell()
-        self._index = lib._get_index(fname,
-                                     reload=rebuild_index)
+        self._index = lib.get_index(fname,
+                                    reload=rebuild_index,
+                                    debug=debug)
         self._reopen(bufsize)
         self.filehead_config = self._read_filehead_config_string()
         self._ens_pos = self._index['pos'][lib._boolarray_firstensemble_ping(
@@ -117,7 +156,7 @@ class _Ad2cpReader():
             self.f.close()
         except AttributeError:
             pass
-        self.f = open(self.fname, 'rb', bufsize)
+        self.f = open(_abspath(self.fname), 'rb', bufsize)
 
     def _read_filehead_config_string(self, ):
         hdr = self._read_hdr()
@@ -266,9 +305,9 @@ class _Ad2cpReader():
 
             elif id in [22, 27, 29, 30, 31, 35, 36]:  # avg record, bt record,
                 # DVL, alt record, avg alt_raw record, raw echo, raw echo transmit
-                warnings.warn(
-                    "Unhandled ID: 0x{:02X} ({:02d})\n"
-                    "    This ID is not yet handled by DOLfYN.\n".format(id, id))
+                if self.debug:
+                    logging.debug(
+                        "Skipped ID: 0x{:02X} ({:02d})\n".format(id, id))
                 self.f.seek(hdr['sz'], 1)
             elif id == 160:
                 # 0xa0 (i.e., 160) is a 'string data record'
@@ -279,7 +318,8 @@ class _Ad2cpReader():
             else:
                 if id not in self.unknown_ID_count:
                     self.unknown_ID_count[id] = 1
-                    print('Unknown ID: 0x{:02X}!'.format(id))
+                    if self.debug:
+                        logging.warning('Unknown ID: 0x{:02X}!'.format(id))
                 else:
                     self.unknown_ID_count[id] += 1
                 self.f.seek(hdr['sz'], 1)
@@ -374,17 +414,17 @@ def _reorg(dat):
         cfg['n_cells' + tag] = tmp['n_cells']
         cfg['coord_sys_axes' + tag] = tmp['cy']
         cfg['n_beams' + tag] = tmp['n_beams']
-        cfg['xmit_energy' + tag] = np.median(dnow['xmit_energy'])
-        cfg['ambig_vel' + tag] = np.median(dnow['ambig_vel'])
+        cfg['ambig_vel' +
+            tag] = lib._collapse(dnow['ambig_vel'], name='ambig_vel')
 
-        for ky in ['SerialNum', 'cell_size', 'blank_dist', 'nom_corr',
-                   'data_desc', 'vel_scale', 'power_level']:
+        for ky in ['SerialNum', 'cell_size', 'blank_dist', 'nominal_corr',
+                   'power_level_dB']:
             cfg[ky + tag] = lib._collapse(dnow[ky],
                                           exclude=collapse_exclude,
                                           name=ky)
 
         for ky in ['c_sound', 'temp', 'pressure', 'heading', 'pitch', 'roll',
-                   'mag', 'accel', 'batt', 'temp_mag', 'temp_clock', 'error',
+                   'mag', 'accel', 'batt', 'temp_clock', 'error',
                    'status', 'ensemble',
                    ]:
             outdat['data_vars'][ky + tag] = dnow[ky]
@@ -393,12 +433,12 @@ def _reorg(dat):
                 outdat['units'][ky + tag] = '#'
 
         for ky in ['vel', 'amp', 'corr', 'prcnt_gd', 'echo', 'dist',
-                   'orientmat', 'angrt', 'quaternion', 'ast_pressure',
+                   'orientmat', 'angrt', 'quaternions', 'ast_pressure',
                    'alt_dist', 'alt_quality', 'alt_status',
                    'ast_dist', 'ast_quality', 'ast_offset_time',
                    'altraw_nsamp', 'altraw_dsamp', 'altraw_samp',
                    'status0', 'fom', 'temp_press', 'press_std',
-                   'pitch_std', 'roll_std', 'heading_std',
+                   'pitch_std', 'roll_std', 'heading_std', 'xmit_energy',
                    ]:
             if ky in dnow:
                 outdat['data_vars'][ky + tag] = dnow[ky]
@@ -413,20 +453,63 @@ def _reorg(dat):
                     ard[grp] = {}
                 ard[ky.rstrip('_ast')] = outdat['data_vars'].pop(ky)
 
-    outdat['attrs']['coord_sys'] = {'XYZ': 'inst',
-                                    'ENU': 'earth',
-                                    'beam': 'beam'}[cfg['coord_sys_axes']]
-    tmp = lib._status2data(outdat['data_vars']['status'])  # returns a dict
+        # Read altimeter status
+        alt_status = lib._alt_status2data(outdat['data_vars']['alt_status'])
+        for ky in alt_status:
+            outdat['attrs'][ky] = lib._collapse(
+                alt_status[ky].astype('uint8'), name=ky)
+        outdat['data_vars'].pop('alt_status')
+
+        # Power level index
+        power = {0: 'high', 1: 'med-high', 2: 'med-low', 3: 'low'}
+        outdat['attrs']['power_level_alt'] = power[outdat['attrs'].pop(
+            'power_level_idx_alt')]
+
+    # Read status data
+    status0_vars = [x for x in outdat['data_vars'] if 'status0' in x]
+    # Status data is the same across all tags, and there is always a 'status' and 'status0'
+    status0_key = status0_vars[0]
+    status0_data = lib._status02data(outdat['data_vars'][status0_key])
+    status_key = status0_key.replace('0', '')
+    status_data = lib._status2data(outdat['data_vars'][status_key])
+
+    # Individual status codes
+    # Wake up state
+    wake = {0: 'bad power', 1: 'power on', 2: 'break', 3: 'clock'}
+    outdat['attrs']['wakeup_state'] = wake[lib._collapse(
+        status_data.pop('wakeup_state'), name=ky)]
 
     # Instrument direction
     # 0: XUP, 1: XDOWN, 2: YUP, 3: YDOWN, 4: ZUP, 5: ZDOWN,
     # 7: AHRS, handle as ZUP
     nortek_orient = {0: 'horizontal', 1: 'horizontal', 2: 'horizontal',
                      3: 'horizontal', 4: 'up', 5: 'down', 7: 'AHRS'}
-    outdat['attrs']['orientation'] = nortek_orient[tmp['orient_up'][0]]
-    orient_status = {0: 'fixed', 1: 'auto_UD', 3: 'AHRS-3D'}
-    outdat['attrs']['orient_status'] = orient_status[tmp['auto orientation'][0]]
+    outdat['attrs']['orientation'] = nortek_orient[lib._collapse(
+        status_data.pop('orient_up'), name='orientation')]
 
+    # Orientation detection
+    orient_status = {0: 'fixed', 1: 'auto_UD', 3: 'AHRS-3D'}
+    outdat['attrs']['orient_status'] = orient_status[lib._collapse(
+        status_data.pop('auto_orientation'), name='orient_status')]
+
+    # Status variables
+    for ky in ['low_volt_skip', 'active_config', 'telemetry_data', 'boost_running']:
+        outdat['data_vars'][ky] = status_data[ky].astype('uint8')
+
+    # Processor idle state - need to save as 1/0 per netcdf attribute limitations
+    for ky in status0_data:
+        outdat['attrs'][ky] = lib._collapse(
+            status0_data[ky].astype('uint8'), name=ky)
+
+    # Remove status0 variables - keep status variables as they useful for finding missing pings
+    [outdat['data_vars'].pop(var) for var in status0_vars]
+
+    # Set coordinate system
+    outdat['attrs']['coord_sys'] = {'XYZ': 'inst',
+                                    'ENU': 'earth',
+                                    'beam': 'beam'}[cfg['coord_sys_axes']]
+
+    # Copy appropriate vars to rotate_vars
     for ky in ['accel', 'angrt', 'mag']:
         for dky in outdat['data_vars'].keys():
             if dky == ky or dky.startswith(ky + '_'):
@@ -438,10 +521,10 @@ def _reorg(dat):
 
 
 def _reduce(data):
-    """This function takes the output from `reorg`, and further simplifies the 
-    data. Mostly this is combining system, environmental, and orientation data 
+    """This function takes the output from `reorg`, and further simplifies the
+    data. Mostly this is combining system, environmental, and orientation data
     --- from different data structures within the same ensemble --- by
-    averaging.  
+    averaging.
     """
     dv = data['data_vars']
     dc = data['coords']
@@ -449,8 +532,7 @@ def _reduce(data):
 
     # Average these fields
     for ky in ['c_sound', 'temp', 'pressure',
-               'temp_press', 'temp_clock', 'temp_mag',
-               'batt']:
+               'temp_press', 'temp_clock', 'batt']:
         lib._reduce_by_average(dv, ky, ky + '_b5')
 
     # Angle-averaging is treated separately
