@@ -73,11 +73,9 @@ def _inst2earth(adcpo, reverse=False, rotate_vars=None, force=False):
 
     _dcheck = rotb._check_rotmat_det(rmat)
     if not _dcheck.all():
-        warnings.warn("Invalid orientation matrix"
-                      " (determinant != 1) at"
-                      " indices: {}."
-                      .format(np.nonzero(~_dcheck)[0]),
-                      UserWarning)
+        warnings.warn("Invalid orientation matrix (determinant != 1) at indices: {}. "
+                      "If rotated, data at these indices will be erroneous."
+                      .format(np.nonzero(~_dcheck)[0]), UserWarning)
 
     # The dictionary of rotation matrices for different sized arrays.
     rmd = {3: rmat, }
@@ -102,6 +100,7 @@ def _inst2earth(adcpo, reverse=False, rotate_vars=None, force=False):
         n = dat.shape[0]
         # Nortek documents sign change for upside-down instruments
         if down:
+            # This is equivalent to adding 180 degrees to roll axis in _calc_omat()
             sign = np.array([1, -1, -1, -1], ndmin=dat.ndim).T
             signIMU = np.array([1, -1, -1], ndmin=dat.ndim).T
             if not reverse:
